#pragma once
#include "AvatarBody.h"
#include "AvatarControllers.h"
#include "AvatarHands.h"
#include "LipSync.h"
#include "Utilities/GlBuffer.h"
#include "Utilities/GlTexture.h"
#include "Utilities/Locked.h"
#include "Utilities/MemoryMap.h"

#include "OVR_Avatar.h"

#include "glm/gtc/quaternion.hpp"
#include "glm/vec3.hpp"

#include <vector>

#ifdef ANDROID
#include <jni.h>
#endif

class Application {
 public:
  enum class HMDType { Rift, RiftS, Quest };
#ifdef ANDROID
  Application(HMDType hmdType, jobject activity, JNIEnv* jniEnv);
#else
  Application(HMDType hmdType);
#endif
  ~Application();
  Application(const Application&) = default;
  Application(Application&&) = default;
  Application& operator=(const Application&) = default;
  Application& operator=(Application&&) = default;

  ovrUtilities::ovrTextureInfo GetEnvTextureInfo();
  void LoadEnvTexture(uint32_t texId, const ovrUtilities::ovrTextureInfo& texInfo);

  struct HandParams {
    bool isActive{false};
    glm::vec3 Position;
    glm::quat Rotation;
    uint32_t ButtonMask;
    uint32_t TouchMask;
    float JoystickX;
    float JoystickY;
    float IndexTrigger;
    float HandTrigger;
  };

  struct FrameParams {
    float DeltaSeconds;
    // These are "center" eye values.
    glm::vec3 EyePos;
    glm::quat EyeRot;
    HandParams Hands[2];
  };
  void Frame(FrameParams& params);

  void Render(uint32_t eye, ovrUtilities::ovrGlBuffer& sceneMatrices);

 private:
  void CommonInit(const char* environmentMap, const char* reflectionMap);

  template <typename CallableT>
  void ProccessPlatformSDKMsgs(const CallableT& callable);
  template <typename CallableT>
  void ProccessAvatarSDKMsgs(const CallableT& callable);

  void PlatformInitializing();
  void CheckingEntitlement();
  void RetrievingAvatarSpec();
  void RetrievingAssets();
  void Running(FrameParams& params);

  void MicCallback(int16_t* buffer);

  enum class ovrAppState {
    PlatformInitializing,
    CheckingEntitlement,
    RetrievingAvatarSpec,
    RetrievingAssets,
    Running
  };

  std::unique_ptr<ovrUtilities::MemoryMap> EnvTextureMap;
  ovrUtilities::ovrGlTexture EnvReflectionMap;
  ovrUtilities::ovrGlBuffer SharedParams;

#ifdef ANDROID
  jobject Activity;
  JNIEnv* JniEnv;
#endif
  HMDType HmdType;

  ovrAppState AppState{ovrAppState::PlatformInitializing};

  uint64_t UserID{0};

  uint64_t PlatformInitializeRequest{0};
  uint64_t EntitlementRequest{0};

  ovrUtilities::Locked<std::vector<int16_t>> MicAudio;
  ovrLipSync LipSync;

  ovrAvatar* Avatar{nullptr};
  std::unique_ptr<ovrAvatarBody> Body;
  std::unique_ptr<ovrAvatarControllers> Controllers;
  std::unique_ptr<ovrAvatarHands> Hands;
};
