#pragma once
#include "OVR_Avatar.h"

#include "Mesh.h"
#include "Utilities/GlTexture.h"

#include "glm/mat4x4.hpp"

#include <array>
#include <vector>

class ovrAvatarBody {
 public:
  ovrAvatarBody(ovrAvatar* avatar);
  ~ovrAvatarBody() = default;
  ovrAvatarBody(const ovrAvatarBody&) = delete;
  ovrAvatarBody(ovrAvatarBody&&) = delete;
  ovrAvatarBody& operator=(const ovrAvatarBody&) = delete;
  ovrAvatarBody& operator=(ovrAvatarBody&&) = delete;

  // Avatar cant be rendered until all of its assets have been loaded
  bool AllAssetsLoaded() const;

  void LoadMesh(const ovrAvatarAsset* asset);
  void LoadTexture(ovrAvatarAssetID id, const ovrAvatarTextureAssetData* tex);

  void Frame();

  void Render(
      uint32_t eye,
      ovrUtilities::ovrGlBuffer& sharedParams,
      ovrUtilities::ovrGlTexture& envMap,
      ovrUtilities::ovrGlBuffer& sceneMatrices);

 private:
  bool MeshLoaded{false};
  // These will correspond to an entry in the texture array. Once loaded
  // set the entry to 0. One all are 0, we are no longer waiting on any textures
  // and this data can be cleared.
  struct TextureToLoad {
    ovrAvatarAssetID Albedo{0};
    ovrAvatarAssetID Normal{0};
    ovrAvatarAssetID Metallic{0};
  };
  std::vector<TextureToLoad> TexturesToLoad;

  ovrAvatarExpressiveParameters ExpressiveParams;

  ovrMesh Mesh;

  ovrUtilities::ovrGlTexture AlbedoTextures;
  ovrUtilities::ovrGlTexture NormalTextures;
  ovrUtilities::ovrGlTexture MetallicTextures;

  uint32_t BlendShapeSize{0};
  // Blend shapes will be stored in a texture buffer.
  // Note, this will only work on PC, Quest, and Go. Not GearVR.
  // Probably better to use a compute shader anyway, but using a texture buffer in the vertex shader
  // is simpler.
  ovrUtilities::ovrGlTexture BShapeTexture;
  ovrUtilities::ovrGlBuffer BShapeTextureBuffer;

  ovrUtilities::ovrGlBuffer MaterialVertexBuffer;
  ovrUtilities::ovrGlBuffer MaterialFragmentBuffer;

  // Body Component cache
  const ovrAvatarBodyComponent* BodyComp = nullptr;
  // Cache the body render part, will use to retrieve blend shape parameters every frame.
  const ovrAvatarRenderPart* CombinedRenderPart = nullptr;
  // Cache the render component too.
  const ovrAvatarComponent* CombinedRenderComp = nullptr;

  // Hold onto our world Transform. Created in Frame, used in Render
  glm::mat4 WorldTransform;
};
