#pragma once
#include "OVR_Avatar.h"

#include "Mesh.h"
#include "Utilities/GlTexture.h"

#include "glm/mat4x4.hpp"

#include <array>
#include <vector>

class ovrAvatarControllers {
 public:
  // CV1Controller true for original Rift, false for RiftS and Quest which use newer touch
  // controller
  ovrAvatarControllers(ovrAvatar* avatar, bool cv1Controller);
  ~ovrAvatarControllers() = default;
  ovrAvatarControllers(const ovrAvatarControllers&) = delete;
  ovrAvatarControllers(ovrAvatarControllers&&) = delete;
  ovrAvatarControllers& operator=(const ovrAvatarControllers&) = delete;
  ovrAvatarControllers& operator=(ovrAvatarControllers&&) = delete;

  // Hands cant be rendered until all of the assets have been loaded
  bool AllAssetsLoaded() const;

  void LoadMesh(ovrAvatarAssetID id, const ovrAvatarAsset* asset);
  void LoadTexture(ovrAvatarAssetID id, const ovrAvatarTextureAssetData* tex);

  void Frame();

  void Render(
      uint32_t eye,
      ovrUtilities::ovrGlBuffer& sharedParams,
      ovrUtilities::ovrGlTexture& envMap,
      ovrUtilities::ovrGlBuffer& sceneMatrices);

 private:
  // otherwise its a PBSV1 render. always V2 on Quest, can be either on PC, V1 for Rift, V2 for
  // RiftS
  bool PBSV2 = false;

  // Will be one entry for each hand, once both are set to 0, all meshes are loaded, clear this
  // array
  std::vector<ovrAvatarAssetID> MeshesToLoad;

  // These will correspond to an entry in the texture array. Once loaded
  // set the entry to 0. One all are 0, we are no longer waiting on any textures
  // and this data can be cleared.
  struct TextureToLoad {
    ovrAvatarAssetID Albedo{0};
    ovrAvatarAssetID Normal{0};
    ovrAvatarAssetID Metallic{0};
  };
  std::vector<TextureToLoad> TexturesToLoad;

  ovrMesh Meshes[2];

  ovrUtilities::ovrGlTexture AlbedoTextures;
  ovrUtilities::ovrGlTexture NormalTextures; // Only used on Quest
  ovrUtilities::ovrGlTexture MetallicTextures;

  ovrUtilities::ovrGlBuffer MaterialVertexBuffer; // will hold bones for both controllers

  // Hold onto our world Transform. Created in Frame, used in Render
  glm::mat4 WorldTransforms[2];

  // Cache the render parts, will use to retrieve bones every frame.
  const ovrAvatarRenderPart* RenderParts[2] = {nullptr, nullptr};
  // Cache the render component too.
  const ovrAvatarComponent* RenderComps[2] = {nullptr, nullptr};
};
