#pragma once
#include "OVR_Avatar.h"

#include "Mesh.h"
#include "Utilities/GlTexture.h"

#include "glm/mat4x4.hpp"

#include <array>
#include <vector>

class ovrAvatarHands {
 public:
  ovrAvatarHands(ovrAvatar* avatar);
  ~ovrAvatarHands() = default;
  ovrAvatarHands(const ovrAvatarHands&) = delete;
  ovrAvatarHands(ovrAvatarHands&&) = delete;
  ovrAvatarHands& operator=(const ovrAvatarHands&) = delete;
  ovrAvatarHands& operator=(ovrAvatarHands&&) = delete;

  // Hands cant be rendered until all of the assets have been loaded
  bool AllAssetsLoaded() const;

  void LoadMesh(ovrAvatarAssetID id, const ovrAvatarAsset* asset);
  void LoadTexture(ovrAvatarAssetID id, const ovrAvatarTextureAssetData* tex);

  void Frame();

  void Render(
      uint32_t eye,
      ovrUtilities::ovrGlBuffer& sharedParams,
      ovrUtilities::ovrGlBuffer& sceneMatrices);

 private:
  // Will be one entry for each hand, once both are set to 0, all meshes are loaded, clear this
  // array
  std::vector<ovrAvatarAssetID> MeshesToLoad;

  // These will correspond to an entry in the texture array. Once loaded
  // set the entry to 0. One all are 0, we are no longer waiting on any textures
  // and this data can be cleared.
  struct TextureToLoad {
    ovrAvatarAssetID Albedo{0};
    ovrAvatarAssetID Normal{0};
  };
  std::vector<TextureToLoad> TexturesToLoad;

  ovrMesh Meshes[2];

  ovrUtilities::ovrGlTexture AlbedoTextures;
  ovrUtilities::ovrGlTexture NormalTextures;

  ovrUtilities::ovrGlBuffer MaterialVertexBuffer; // will hold bones for both hands
  glm::vec4 AlbedoTints[2]; // one for each hand, just using push constants for these

  // Hold onto our world Transform. Created in Frame, used in Render
  glm::mat4 WorldTransforms[2];

  // Cache the render parts, will use to retrieve bones every frame.
  const ovrAvatarRenderPart* RenderParts[2] = {nullptr, nullptr};
  // Cache the render component too.
  const ovrAvatarComponent* RenderComps[2] = {nullptr, nullptr};
};
