#pragma once
#include "OVR_Avatar.h"

#include "glm/mat4x4.hpp"

#include <cinttypes>
#include <vector>

class ovrMesh {
 public:
  ovrMesh() = default;
  ovrMesh(const ovrAvatarAsset* asset, const ovrAvatarMeshAssetData* mesh);
  ovrMesh(const ovrAvatarAsset* asset, const ovrAvatarMeshAssetDataV2* mesh); // combined mesh
  ~ovrMesh();
  ovrMesh(const ovrMesh&) = delete;
  ovrMesh(ovrMesh&&);
  ovrMesh& operator=(const ovrMesh&) = delete;
  ovrMesh& operator=(ovrMesh&&);

  uint32_t GetVertexArray() const;
  uint32_t GetIndexCount() const;
  const std::vector<glm::mat4>& GetInverseBindPose() const;

 private:
  template <typename T>
  void Init(const ovrAvatarAsset* asset, const T* mesh);

  uint32_t VertexBuffer{0};
  uint32_t IndexBuffer{0};
  uint32_t VertexArray{0};
  uint32_t IndexCount{0};

  std::vector<glm::mat4> InverseBindPose;
};

inline uint32_t ovrMesh::GetVertexArray() const {
  return VertexArray;
}

inline uint32_t ovrMesh::GetIndexCount() const {
  return IndexCount;
}

inline const std::vector<glm::mat4>& ovrMesh::GetInverseBindPose() const {
  return InverseBindPose;
}
