#pragma once
#include "Utilities/GlProgram.h"

#include "glm/vec3.hpp"

enum class UniformIndexBody {
  VertMaterials, // UniformBuffer
  FragMaterials, // UniformBuffer
  SharedParms, // UniformBuffer
  Albedo, // Texture
  Normal, // Texture
  Metallic, // Texture
  EnvironmentMap, // Texture
  BlendShapes // TextureBuffer
};

enum class UniformIndexHand {
  VertMaterials, // UniformBuffer
  SharedParms, // UniformBuffer
  Albedo, // Texture
  Normal, // Texture
  AlbedoTint, // vec4
  Hand, // int
};

enum class UniformIndexController {
  // Params shared by both V1 and V2
  VertMaterials, // UniformBuffer
  SharedParms, // UniformBuffer
  Albedo, // Texture
  Metallic, // Texture
  EnvironmentMap, // Texture
  Hand, // int
  // V2 only Params
  Normal, // Texture
};

#pragma pack(push, 1)
// std140 layout, must match shader
struct ovrSharedParams {
  glm::vec3 ViewPosition;
  float padding1;
  glm::vec3 LightPosition;
  float padding2;
};
#pragma pack(pop)

const ovrUtilities::ovrGlProgram& GetBodyProgram();
const ovrUtilities::ovrGlProgram& GetHandProgram();
const ovrUtilities::ovrGlProgram& GetControllerProgramV1();
const ovrUtilities::ovrGlProgram& GetControllerProgramV2();
