#pragma once

#include <cstdint>

namespace ovrUtilities {

enum ovrGlBufferType_t {
  GLBUFFER_TYPE_UNIFORM,
  GLBUFFER_TYPE_TEXTURE_BUFFER,
  GLBUFFER_TYPE_SHADER_STORAGE,
  GLBUFFER_TYPE_ARRAY,
};

class ovrGlBuffer {
 public:
  ovrGlBuffer();
  ~ovrGlBuffer();
  ovrGlBuffer(const ovrGlBufferType_t type, const size_t dataSize);
  ovrGlBuffer(const ovrGlBuffer&) = delete;
  ovrGlBuffer(ovrGlBuffer&& other);
  ovrGlBuffer& operator=(const ovrGlBuffer&) = delete;
  ovrGlBuffer& operator=(ovrGlBuffer&& other);

  void Bind() const;
  void Unbind() const;

  void Bind(uint32_t index) const;
  void BindRange(uint32_t index, uint32_t offset, uint32_t size) const;
  void Unbind(uint32_t index) const;

  // Bind this buffer as if it was a Shader storage buffer, for use
  // as input/output of a compute shader
  void BindAsSSBO(uint32_t index) const;
  void UnbindAsSSBO(uint32_t index) const;

  template <typename T>
  T* Map() const {
    return (T*)Map();
  }
  void Unmap() const;

  uint32_t GetGlID() const {
    return Buffer;
  }
  uint32_t GetGlTarget() const;

  bool IsValid() const {
    return Buffer != 0;
  }

 private:
  void* Map() const;
  void Destroy();

  uint32_t Target{0};
  uint32_t Buffer{0};
  size_t Size{0};
};

inline uint32_t ovrGlBuffer::GetGlTarget() const {
  return Target;
}

} // namespace ovrUtilities
