#pragma once

#include "GlBuffer.h"
#include "GlCore.h"

namespace ovrUtilities {

class ovrGlTexture {
 public:
  // enum class Type { TwoD, Cube };

  ovrGlTexture() = default;
  // Take ownership of an existing texture
  ovrGlTexture(uint32_t texture, uint32_t target);
  // Create a new texture
  ovrGlTexture(uint32_t target, uint32_t format, uint32_t width, uint32_t height, uint32_t mips);
  // Create a new array texture
  ovrGlTexture(
      uint32_t target,
      uint32_t format,
      uint32_t width,
      uint32_t height,
      uint32_t depth,
      uint32_t mips);
  // creates a texture buffer, backed by the provided ovrGlBuffer
  ovrGlTexture(ovrGlBuffer& buffer, uint32_t format);
  ~ovrGlTexture();
  ovrGlTexture(const ovrGlTexture&) = delete;
  ovrGlTexture(ovrGlTexture&& other);
  ovrGlTexture& operator=(const ovrGlTexture&) = delete;
  ovrGlTexture& operator=(ovrGlTexture&& other);

  bool IsValid() const;
  template <typename Callable_t>
  void Bind(Callable_t callable);

  uint32_t GetGlID() const;
  uint32_t GetGlTarget() const;

  void Activate(uint32_t binding);
  void DeActivate(uint32_t binding);

 private:
  void Free();

  uint32_t Texture{0};
  uint32_t Target{0};
};

inline bool ovrGlTexture::IsValid() const {
  return Texture != 0;
}

template <typename Callable_t>
void ovrGlTexture::Bind(Callable_t callable) {
  glBindTexture(Target, Texture);
  callable(Target);
  glBindTexture(Target, 0);
}

inline uint32_t ovrGlTexture::GetGlID() const {
  return Texture;
}

inline uint32_t ovrGlTexture::GetGlTarget() const {
  return Target;
}

struct ovrTextureInfo {
  uint32_t GlFormat{0};
  uint32_t GlInternalFormat{0};
  uint32_t Width{0};
  uint32_t Height{0};
  uint32_t NumFaces{1}; // 1 or 6 for cubemaps
  uint32_t NumMips{1};
  uint32_t OffsetToData{0}; // Offset to beginning of texture data
};

ovrTextureInfo GetKTXTextureInfo(const uint8_t* data, size_t dataSize);

void LoadKTXTexture(const uint8_t* data, size_t dataSize, const ovrTextureInfo& info);

ovrGlTexture LoadKTXTexture(const uint8_t* data, size_t dataSize);

} // namespace ovrUtilities
