#include "Log.h"

#ifdef ANDROID
#include <sys/syscall.h> // for syscall()
#include <unistd.h> // for gettid()
#else // windows
#define NOMINMAX // stop Windows.h from redefining min and max and breaking std::min / std::max
#include <windows.h> // OutputDebugString
#endif

#include <assert.h>
#include <errno.h>
#include <stdarg.h>
#include <string.h>

void FilePathToTag(const char* filePath, char* strippedTag, size_t const strippedTagSize) {
  // scan backwards from the end to the first slash
  const int len = static_cast<int>(strlen(filePath));
  int slash;
  for (slash = len - 1; slash > 0 && filePath[slash] != '/' && filePath[slash] != '\\'; slash--) {
  }
  if (filePath[slash] == '/' || filePath[slash] == '\\') {
    slash++;
  }
  // copy forward until a dot or 0
  size_t i;

  for (i = 0; i < strippedTagSize - 1; i++) {
    const char c = filePath[slash + i];
    if (c == '.' || c == 0) {
      break;
    }
    strippedTag[i] = c;
  }
  strippedTag[i] = 0;
}

// Log with an explicit tag
void LogWithTag(const int prio, const char* tag, const char* fmt, ...) {
  // fileTag will be something like "jni/App.cpp", which we
  // want to strip down to just "App"
  char strippedTag[128];

  FilePathToTag(tag, strippedTag, sizeof(strippedTag));

  va_list ap;
  va_start(ap, fmt);
  char buffer[4096];
  vsnprintf(buffer, sizeof(buffer), fmt, ap);
#ifdef ANDROID
  __android_log_write(prio, strippedTag, buffer);
#else
  strcat_s(buffer, "\n");
  OutputDebugString(buffer);
#endif
  va_end(ap);
}
