#include "MemoryMap.h"

#include <android/asset_manager.h>
#include <android/asset_manager_jni.h>

using namespace ovrUtilities;

namespace {

class MemoryMapImpl : public MemoryMap {
 public:
  MemoryMapImpl(const char* fileName);
  virtual ~MemoryMapImpl();
  MemoryMapImpl(const MemoryMapImpl&) = delete;
  MemoryMapImpl(MemoryMapImpl&& other) = delete;
  MemoryMapImpl& operator=(const MemoryMapImpl&) = delete;
  MemoryMapImpl& operator=(MemoryMapImpl&& other) = delete;

  const void* data() const override {
    return Data;
  }
  uint32_t size() const override {
    return Size;
  }

 private:
  AAsset* Asset;
  const void* Data;
  uint32_t Size;
};

struct MemMapper {
  AAssetManager* AssetManager{nullptr};
};

MemMapper& GetMapper() {
  static MemMapper mapper;
  return mapper;
}
} // namespace

void SetAssetManager(JNIEnv* env, jobject assetManager) {
  GetMapper().AssetManager = AAssetManager_fromJava(env, assetManager);
}

MemoryMapImpl::MemoryMapImpl(const char* fileName) {
  Asset = AAssetManager_open(GetMapper().AssetManager, fileName, AASSET_MODE_UNKNOWN);
  Data = AAsset_getBuffer(Asset);
  Size = AAsset_getLength(Asset);
}

MemoryMapImpl::~MemoryMapImpl() {
  AAsset_close(Asset);
}

std::unique_ptr<MemoryMap> ovrUtilities::CreateMemoryMap(const char* fileName) {
  return std::make_unique<MemoryMapImpl>(fileName);
}
