#include "MemoryMap.h"

#define WIN32_LEAN_AND_MEAN
#define NOMINMAX
#include <windows.h>

using namespace ovrUtilities;

namespace {

class MemoryMapImpl : public MemoryMap {
 public:
  MemoryMapImpl(HANDLE fileHandle, std::size_t fileSize);
  virtual ~MemoryMapImpl();
  MemoryMapImpl(const MemoryMapImpl&) = delete;
  MemoryMapImpl(MemoryMapImpl&& other) = delete;
  MemoryMapImpl& operator=(const MemoryMapImpl&) = delete;
  MemoryMapImpl& operator=(MemoryMapImpl&& other) = delete;

  const void* data() const override {
    return Data;
  }
  uint32_t size() const override {
    return Size;
  }

 private:
  HANDLE FileHandle;
  HANDLE FileMapping;
  const void* Data;
  uint32_t Size;
};
} // namespace

MemoryMapImpl::MemoryMapImpl(HANDLE fileHandle, std::size_t fileSize)
    : FileHandle(fileHandle), Size((uint32_t)fileSize) {
  FileMapping = CreateFileMapping(FileHandle, nullptr, PAGE_READONLY, 0, 0, nullptr);
  Data = MapViewOfFile(FileMapping, FILE_MAP_READ, 0, 0, 0);
}

MemoryMapImpl::~MemoryMapImpl() {
  UnmapViewOfFile(Data);
  CloseHandle(FileMapping);
  CloseHandle(FileHandle);
}

std::unique_ptr<MemoryMap> ovrUtilities::CreateMemoryMap(const char* fileName) {
  auto handle =
      CreateFile(fileName, GENERIC_READ, 0, nullptr, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, 0);
  if (handle == INVALID_HANDLE_VALUE)
    return nullptr;
  LARGE_INTEGER fileSize;
  if (!GetFileSizeEx(handle, &fileSize))
    return nullptr;
  if (fileSize.QuadPart == 0)
    return nullptr;

  return std::make_unique<MemoryMapImpl>(handle, static_cast<size_t>(fileSize.QuadPart));
}
