#include "VertexBufferWriter.h"

#include <algorithm>
#include <cmath>

using namespace std;

template <typename T>
void VertexBufferWriter::Write(const T& arg) {
  auto offset = Buffer.size();
  Buffer.resize(offset + sizeof(arg));
  memcpy(&Buffer[offset], &arg, sizeof(arg));
}

void VertexBufferWriter::PackPosition(float x, float y, float z) {
  Write(x);
  Write(y);
  Write(z);
}

void VertexBufferWriter::PackFrenet(float x, float y, float z) {
  x = clamp<float>(x * SHRT_MAX, SHRT_MIN, SHRT_MAX);
  y = clamp<float>(y * SHRT_MAX, SHRT_MIN, SHRT_MAX);
  z = clamp<float>(z * SHRT_MAX, SHRT_MIN, SHRT_MAX);

  int32_t i0 = (int32_t)x;
  int32_t i1 = (int32_t)y;
  int32_t i2 = (int32_t)z;
  const int32_t bitmask = 0xffc0; // 10 bits

  int32_t packed =
      static_cast<int32_t>(((i2 & bitmask) << 14) | ((i1 & bitmask) << 4) | ((i0 & bitmask) >> 6));

  Write(packed);
}

void VertexBufferWriter::PackUV(float u, float v) {
  Write(u);
  Write(v);
}

void VertexBufferWriter::PackJointIndices(uint8_t index1, uint8_t index2) {
  Write(index1);
  Write(index2);
}

void VertexBufferWriter::PackJointWeights(float weight1, float weight2) {
  weight1 = clamp(weight1, 0.0f, 1.0f) * 255.0f;
  weight2 = clamp(weight2, 0.0f, 1.0f) * 255.0f;
  Write(static_cast<uint8_t>(weight1));
  Write(static_cast<uint8_t>(weight2));
}

void VertexBufferWriter::PackRenderPart(float part) {
  // part is 0-1, but it represents renderparts 0-4. We will pass as an integer
  part *= 4.0f;
  part = clamp(roundf(part), 0.0f, 4.0f);
  Write(static_cast<uint8_t>(part));
  // padding
  Write(static_cast<uint8_t>(0));
  Write(static_cast<uint8_t>(0));
  Write(static_cast<uint8_t>(0));
}
