#pragma once

#include "OVR_Avatar.h"

#include <vector>

class VertexBufferWriter {
 public:
  VertexBufferWriter() = default;
  ~VertexBufferWriter() = default;
  VertexBufferWriter(const VertexBufferWriter&) = delete;
  VertexBufferWriter(VertexBufferWriter&&) = delete;
  VertexBufferWriter& operator=(const VertexBufferWriter&) = delete;
  VertexBufferWriter& operator=(VertexBufferWriter&&) = delete;

  const uint8_t* data() const {
    return std::data(Buffer);
  }

  size_t size() const {
    return std::size(Buffer);
  }

  // Pack position into 3 32bit floats, 12 bytes total
  void PackPosition(float x, float y, float z);

  // Pack normals/tangents into 10_10_10_2 format, 4 bytes total
  void PackFrenet(float x, float y, float z);

  // Pack uv's into 2 32 bit floats, 8 bytes total
  void PackUV(float u, float v);

  // packed into 2 8bit ints, 2 bytes total
  void PackJointIndices(uint8_t index1, uint8_t index2);

  // packed into 2 8bit fixed point floats, 2 bytes total
  void PackJointWeights(float weight1, float weight2);

  // packed into an 8bit integer, with 3 extra unused 8bit integers, for padding. Used as index into
  // texture array and material buffer.
  void PackRenderPart(float part);

 private:
  template <typename T>
  void Write(const T& arg);

  std::vector<uint8_t> Buffer;
};
